# Copyright 2017 OVH SAS
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from openstack import exceptions
from openstack.network.v2 import subnet as _subnet
from openstack.tests.unit import base


class TestSubnet(base.TestCase):
    network_name = 'network_name'
    subnet_name = 'subnet_name'
    subnet_id = '1f1696eb-7f47-47f6-835c-4889bff88604'
    subnet_cidr = '192.168.199.0/24'
    subnetpool_cidr = '172.16.0.0/28'
    prefix_length = 28

    mock_network_rep = {
        'id': '881d1bb7-a663-44c0-8f9f-ee2765b74486',
        'name': network_name,
    }

    mock_subnet_rep = {
        'allocation_pools': [
            {'start': '192.168.199.2', 'end': '192.168.199.254'}
        ],
        'cidr': subnet_cidr,
        'created_at': '2017-04-24T20:22:23Z',
        'description': '',
        'dns_nameservers': [],
        'enable_dhcp': False,
        'gateway_ip': '192.168.199.1',
        'host_routes': [],
        'id': subnet_id,
        'ip_version': 4,
        'ipv6_address_mode': None,
        'ipv6_ra_mode': None,
        'name': subnet_name,
        'network_id': mock_network_rep['id'],
        'project_id': '861808a93da0484ea1767967c4df8a23',
        'revision_number': 2,
        'service_types': [],
        'subnetpool_id': None,
        'tags': [],
    }

    mock_subnetpool_rep = {
        'id': 'f49a1319-423a-4ee6-ba54-1d95a4f6cc68',
        'prefixes': ['172.16.0.0/16'],
    }

    def _compare_subnets(self, exp, real):
        self.assertDictEqual(
            _subnet.Subnet(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def test_get_subnet(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets'],
                        qs_elements=[f'name={self.subnet_name}'],
                    ),
                    json={'subnets': [self.mock_subnet_rep]},
                ),
            ]
        )
        r = self.cloud.get_subnet(self.subnet_name)
        self.assertIsNotNone(r)
        self._compare_subnets(self.mock_subnet_rep, r)
        self.assert_calls()

    def test_get_subnet_by_id(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json={'subnet': self.mock_subnet_rep},
                )
            ]
        )
        r = self.cloud.get_subnet_by_id(self.subnet_id)
        self.assertIsNotNone(r)
        self._compare_subnets(self.mock_subnet_rep, r)
        self.assert_calls()

    def test_create_subnet(self):
        pool = [{'start': '192.168.199.2', 'end': '192.168.199.254'}]
        dns = ['8.8.8.8']
        routes = [{"destination": "0.0.0.0/0", "nexthop": "123.456.78.9"}]
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['host_routes'] = routes
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'cidr': self.subnet_cidr,
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                                'allocation_pools': pool,
                                'dns_nameservers': dns,
                                'host_routes': routes,
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name,
            self.subnet_cidr,
            allocation_pools=pool,
            dns_nameservers=dns,
            host_routes=routes,
        )
        self._compare_subnets(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_string_ip_version(self):
        '''Allow ip_version as a string'''
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': self.mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'cidr': self.subnet_cidr,
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name, self.subnet_cidr, ip_version='4'
        )
        self._compare_subnets(self.mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_bad_ip_version(self):
        '''String ip_versions must be convertable to int'''
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
            ]
        )
        with self.assertRaises(
            exceptions.SDKException, msg="ip_version must be an integer"
        ):
            self.cloud.create_subnet(
                self.network_name, self.subnet_cidr, ip_version='4x'
            )
        self.assert_calls()

    def test_create_subnet_without_gateway_ip(self):
        pool = [{'start': '192.168.199.2', 'end': '192.168.199.254'}]
        dns = ['8.8.8.8']
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['gateway_ip'] = None
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'cidr': self.subnet_cidr,
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                                'allocation_pools': pool,
                                'gateway_ip': None,
                                'dns_nameservers': dns,
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name,
            self.subnet_cidr,
            allocation_pools=pool,
            dns_nameservers=dns,
            disable_gateway_ip=True,
        )
        self._compare_subnets(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_with_gateway_ip(self):
        pool = [{'start': '192.168.199.8', 'end': '192.168.199.254'}]
        gateway = '192.168.199.2'
        dns = ['8.8.8.8']
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['gateway_ip'] = gateway
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'cidr': self.subnet_cidr,
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                                'allocation_pools': pool,
                                'gateway_ip': gateway,
                                'dns_nameservers': dns,
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name,
            self.subnet_cidr,
            allocation_pools=pool,
            dns_nameservers=dns,
            gateway_ip=gateway,
        )
        self._compare_subnets(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_conflict_gw_ops(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', 'kooky'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=['name=kooky'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
            ]
        )
        gateway = '192.168.200.3'
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_subnet,
            'kooky',
            self.subnet_cidr,
            gateway_ip=gateway,
            disable_gateway_ip=True,
        )
        self.assert_calls()

    def test_create_subnet_bad_network(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', 'duck'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=['name=duck'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_subnet,
            'duck',
            self.subnet_cidr,
        )
        self.assert_calls()

    def test_create_subnet_non_unique_network(self):
        net1 = dict(id='123', name=self.network_name)
        net2 = dict(id='456', name=self.network_name)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [net1, net2]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_subnet,
            self.network_name,
            self.subnet_cidr,
        )
        self.assert_calls()

    def test_create_subnet_from_subnetpool_with_prefixlen(self):
        pool = [{'start': '172.16.0.2', 'end': '172.16.0.15'}]
        id = '143296eb-7f47-4755-835c-488123475604'
        gateway = '172.16.0.1'
        dns = ['8.8.8.8']
        routes = [{"destination": "0.0.0.0/0", "nexthop": "123.456.78.9"}]
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['host_routes'] = routes
        mock_subnet_rep['gateway_ip'] = gateway
        mock_subnet_rep['subnetpool_id'] = self.mock_subnetpool_rep['id']
        mock_subnet_rep['cidr'] = self.subnetpool_cidr
        mock_subnet_rep['id'] = id
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                                'allocation_pools': pool,
                                'dns_nameservers': dns,
                                'use_default_subnetpool': True,
                                'prefixlen': self.prefix_length,
                                'host_routes': routes,
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name,
            allocation_pools=pool,
            dns_nameservers=dns,
            use_default_subnetpool=True,
            prefixlen=self.prefix_length,
            host_routes=routes,
        )
        mock_subnet_rep.update(
            {'prefixlen': self.prefix_length, 'use_default_subnetpool': True}
        )
        self._compare_subnets(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_from_specific_subnetpool(self):
        pool = [{'start': '172.16.0.2', 'end': '172.16.0.15'}]
        id = '143296eb-7f47-4755-835c-488123475604'
        gateway = '172.16.0.1'
        dns = ['8.8.8.8']
        routes = [{"destination": "0.0.0.0/0", "nexthop": "123.456.78.9"}]
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['host_routes'] = routes
        mock_subnet_rep['gateway_ip'] = gateway
        mock_subnet_rep['subnetpool_id'] = self.mock_subnetpool_rep['id']
        mock_subnet_rep['cidr'] = self.subnetpool_cidr
        mock_subnet_rep['id'] = id
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', self.network_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=[f'name={self.network_name}'],
                    ),
                    json={'networks': [self.mock_network_rep]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=[
                            'v2.0',
                            'subnetpools',
                            self.mock_subnetpool_rep['id'],
                        ],
                    ),
                    json={"subnetpool": self.mock_subnetpool_rep},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnet': mock_subnet_rep},
                    validate=dict(
                        json={
                            'subnet': {
                                'enable_dhcp': False,
                                'ip_version': 4,
                                'network_id': self.mock_network_rep['id'],
                                'allocation_pools': pool,
                                'dns_nameservers': dns,
                                'subnetpool_id': self.mock_subnetpool_rep[
                                    'id'
                                ],
                                'prefixlen': self.prefix_length,
                                'host_routes': routes,
                            }
                        }
                    ),
                ),
            ]
        )
        subnet = self.cloud.create_subnet(
            self.network_name,
            allocation_pools=pool,
            dns_nameservers=dns,
            subnetpool_name_or_id=self.mock_subnetpool_rep['id'],
            prefixlen=self.prefix_length,
            host_routes=routes,
        )
        mock_subnet_rep.update(
            {'prefixlen': self.prefix_length, 'use_default_subnetpool': None}
        )
        self._compare_subnets(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_delete_subnet(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets'],
                        qs_elements=[f'name={self.subnet_name}'],
                    ),
                    json={'subnets': [self.mock_subnet_rep]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json={},
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_subnet(self.subnet_name))
        self.assert_calls()

    def test_delete_subnet_not_found(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', 'goofy'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets'],
                        qs_elements=['name=goofy'],
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.assertFalse(self.cloud.delete_subnet('goofy'))
        self.assert_calls()

    def test_delete_subnet_multiple_found(self):
        subnet1 = dict(id='123', name=self.subnet_name)
        subnet2 = dict(id='456', name=self.subnet_name)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_name],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets'],
                        qs_elements=[f'name={self.subnet_name}'],
                    ),
                    json={'subnets': [subnet1, subnet2]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.delete_subnet,
            self.subnet_name,
        )
        self.assert_calls()

    def test_delete_subnet_using_id(self):
        subnet1 = dict(id='123', name=self.subnet_name)
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', subnet1['id']],
                    ),
                    json=subnet1,
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', subnet1['id']],
                    ),
                    json={},
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_subnet(subnet1['id']))
        self.assert_calls()

    def test_update_subnet(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        expected_subnet['name'] = 'goofy'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json=self.mock_subnet_rep,
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json={'subnet': expected_subnet},
                    validate=dict(json={'subnet': {'name': 'goofy'}}),
                ),
            ]
        )
        subnet = self.cloud.update_subnet(self.subnet_id, subnet_name='goofy')
        self._compare_subnets(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_gateway_ip(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        gateway = '192.168.199.3'
        expected_subnet['gateway_ip'] = gateway
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json=self.mock_subnet_rep,
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json={'subnet': expected_subnet},
                    validate=dict(json={'subnet': {'gateway_ip': gateway}}),
                ),
            ]
        )
        subnet = self.cloud.update_subnet(self.subnet_id, gateway_ip=gateway)
        self._compare_subnets(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_disable_gateway_ip(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        expected_subnet['gateway_ip'] = None
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json=self.mock_subnet_rep,
                ),
                dict(
                    method='PUT',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'subnets', self.subnet_id],
                    ),
                    json={'subnet': expected_subnet},
                    validate=dict(json={'subnet': {'gateway_ip': None}}),
                ),
            ]
        )
        subnet = self.cloud.update_subnet(
            self.subnet_id, disable_gateway_ip=True
        )
        self._compare_subnets(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_conflict_gw_ops(self):
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.update_subnet,
            self.subnet_id,
            gateway_ip="192.168.199.3",
            disable_gateway_ip=True,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions as sdk_exc
from openstack.image.v2 import image as _image
from openstack.image.v2 import member as _member
from openstack.tests.functional.image.v2 import base


TEST_IMAGE_NAME = 'Test Image for Sharing'
MEMBER_STATUS_PENDING = 'pending'
MEMBER_STATUS_ACCEPTED = 'accepted'


class TestImageMember(base.BaseImageTest):
    def setUp(self):
        super().setUp()

        # NOTE(jbeen): 1-byte dummy image data for sharing tests; not bootable.
        self.image = self.admin_image_client.create_image(
            name=TEST_IMAGE_NAME,
            disk_format='raw',
            container_format='bare',
            visibility='shared',
            data=b'0',
        )
        self.assertIsInstance(self.image, _image.Image)
        self.assertEqual(TEST_IMAGE_NAME, self.image.name)

        member_id = self.user_cloud.session.get_project_id()
        self.assertIsNotNone(member_id)
        assert member_id is not None
        self.member_id = member_id

    def tearDown(self):
        self.admin_image_client.delete_image(self.image)
        self.admin_image_client.wait_for_delete(self.image)

        super().tearDown()

    def test_image_members(self):
        # add member
        member = self.admin_image_client.add_member(
            image=self.image, member=self.member_id
        )
        self.assertIsInstance(member, _member.Member)
        self.assertEqual(self.member_id, member.member_id)
        self.assertEqual(MEMBER_STATUS_PENDING, member.status)

        # get member
        member = self.admin_image_client.get_member(
            image=self.image, member=self.member_id
        )
        self.assertIsInstance(member, _member.Member)
        self.assertEqual(self.member_id, member.member_id)

        # list members
        members = list(self.admin_image_client.members(image=self.image))
        self.assertIn(self.member_id, {m.id for m in members})

        # update member
        member = self.image_client.update_member(
            image=self.image,
            member=self.member_id,
            status=MEMBER_STATUS_ACCEPTED,
        )
        self.assertIsInstance(member, _member.Member)
        self.assertEqual(self.member_id, member.member_id)
        self.assertEqual(MEMBER_STATUS_ACCEPTED, member.status)

        # remove member
        self.admin_image_client.remove_member(
            image=self.image, member=self.member_id
        )
        self.assertRaises(
            sdk_exc.NotFoundException,
            self.admin_image_client.get_member,
            image=self.image,
            member=self.member_id,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.shared_file_system.v2 import share_group as _share_group
from openstack.tests.functional.shared_file_system.v2 import base


class ShareGroupTest(base.BaseSharedFileSystemTest):
    def setUp(self):
        super().setUp()

        self.SHARE_GROUP_NAME = self.getUniqueString()
        share_grp = self.user_cloud.shared_file_system.create_share_group(
            name=self.SHARE_GROUP_NAME
        )
        self.user_cloud.shared_file_system.wait_for_status(
            share_grp,
            status='available',
            failures=['error'],
            interval=5,
            wait=self._wait_for_timeout,
        )
        self.assertIsNotNone(share_grp)
        self.assertIsNotNone(share_grp.id)
        self.SHARE_GROUP_ID = share_grp.id

    def test_get(self):
        sot = self.user_cloud.shared_file_system.get_share_group(
            self.SHARE_GROUP_ID
        )
        assert isinstance(sot, _share_group.ShareGroup)
        self.assertEqual(self.SHARE_GROUP_ID, sot.id)

    def test_find(self):
        sot = self.user_cloud.shared_file_system.find_share_group(
            self.SHARE_GROUP_NAME
        )
        assert isinstance(sot, _share_group.ShareGroup)
        self.assertEqual(self.SHARE_GROUP_NAME, sot.name)
        self.assertEqual(self.SHARE_GROUP_ID, sot.id)

    def test_list_delete_share_group(self):
        s_grps = self.user_cloud.shared_file_system.share_groups()
        self.assertGreater(len(list(s_grps)), 0)
        for s_grp in s_grps:
            for attribute in ('id', 'name', 'created_at'):
                self.assertTrue(hasattr(s_grp, attribute))

            sot = self.operator_cloud.shared_file_system.delete_share_group(
                s_grp
            )
            self.assertIsNone(sot)

    def test_update(self):
        u_gp = self.user_cloud.shared_file_system.update_share_group(
            self.SHARE_GROUP_ID, description='updated share group'
        )
        get_u_gp = self.user_cloud.shared_file_system.get_share_group(u_gp.id)
        self.assertEqual('updated share group', get_u_gp.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import quota_set as _quota_set
from openstack.tests.functional import base


class TestQuotaSet(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        if not self.operator_cloud:
            self.skipTest("Operator cloud must be set for this test")

        self.project = self.create_temporary_project()

    def test_quota_set(self):
        # update quota

        quota_set = self.operator_cloud.block_storage.update_quota_set(
            self.project.id, volumes=123
        )
        self.assertIsInstance(quota_set, _quota_set.QuotaSet)
        self.assertEqual(quota_set.volumes, 123)

        # retrieve details of the (updated) quota

        quota_set = self.operator_cloud.block_storage.get_quota_set(
            self.project.id
        )
        self.assertIsInstance(quota_set, _quota_set.QuotaSet)
        self.assertEqual(quota_set.volumes, 123)

        # retrieve quota defaults

        defaults = self.operator_cloud.block_storage.get_quota_set_defaults(
            self.project.id
        )
        self.assertIsInstance(defaults, _quota_set.QuotaSet)
        self.assertNotEqual(defaults.volumes, 123)

        # revert quota

        self.operator_cloud.block_storage.revert_quota_set(self.project.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.shared_file_system.v2 import share_type as _share_type
from openstack.tests.functional.shared_file_system.v2 import base


class ShareTypeTest(base.BaseSharedFileSystemTest):
    def setUp(self):
        super().setUp()

        self.SHARE_TYPE_NAME = self.getUniqueString()
        stype = self.create_share_type(
            self.SHARE_TYPE_NAME,
            extra_specs={
                'driver_handles_share_servers': False,
                'snapshot_support': True,
                'delete': True,
            },
            description=None,
        )
        self.assertIsNotNone(stype)
        self.assertIsNotNone(stype.id)
        self.SHARE_TYPE_ID = stype.id

    def test_get_share_type(self):
        sot = self.operator_cloud.shared_file_system.get_share_type(
            self.SHARE_TYPE_ID
        )
        assert isinstance(sot, _share_type.ShareType)
        self.assertEqual(self.SHARE_TYPE_ID, sot.id)
        self.assertEqual(
            {
                'driver_handles_share_servers': 'False',
                'snapshot_support': 'True',
                'delete': 'True',
            },
            sot.extra_specs,
        )

    def test_list_share_type(self):
        share_types = self.user_cloud.shared_file_system.share_types()
        self.assertGreater(len(list(share_types)), 0)
        for share_type in share_types:
            for attribute in ('id', 'name', 'is_default', 'description'):
                self.assertTrue(hasattr(share_type, attribute))

    def test_delete_share_type(self):
        sot = self.operator_cloud.shared_file_system.delete_share_type(
            self.SHARE_TYPE_ID
        )
        self.assertIsNone(sot)

    def test_update_share_type(self):
        u_type = self.operator_cloud.shared_file_system.update_share_type(
            self.SHARE_TYPE_ID, description='updated share type'
        )
        get_u_type = self.operator_cloud.shared_file_system.get_share_type(
            u_type.id
        )
        self.assertEqual('updated share type', get_u_type.description)

    def test_update_share_type_extra_specs(self):
        sot = self.operator_cloud.shared_file_system.get_share_type(
            self.SHARE_TYPE_ID
        )
        sot_set = self.operator_cloud.shared_file_system.update_share_type_extra_specs(  # noqa: E501
            sot['id'], foo='bar', baz='qux'
        )

        expected_extra_specs = {
            'foo': 'bar',
            'baz': 'qux',
        }
        self.assertEqual(expected_extra_specs, sot_set['extra_specs'])
        self.assertIn('extra_specs', sot_set)

    def test_delete_share_type_extra_spec_property(self):
        sot = self.operator_cloud.shared_file_system.get_share_type(
            self.SHARE_TYPE_ID
        )
        self.operator_cloud.shared_file_system.delete_share_type_extra_spec_property(
            sot['id'], 'delete'
        )
        expected_extra_specs = {
            'driver_handles_share_servers': 'False',
            'snapshot_support': 'True',
        }
        sot = self.operator_cloud.shared_file_system.get_share_type(
            self.SHARE_TYPE_ID
        )
        self.assertEqual(expected_extra_specs, sot.extra_specs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter

from openstack.dns.v2 import _base
from openstack import resource


class ZoneImport(_base.Resource):
    """DNS Zone Import Resource"""

    resource_key = ''
    resources_key = 'imports'
    base_path = '/zones/tasks/imports'

    create_opts = resource.CreateOpts(request_key=None)

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters('zone_id', 'message', 'status')

    #: Properties
    #: Timestamp when the zone was created
    created_at = resource.Body('created_at')
    #: Links contains a `self` pertaining to this zone or a `next` pertaining
    #: to next page
    links = resource.Body('links', type=dict)
    #: Message
    message = resource.Body('message')
    #: Returns the total_count of resources matching this filter
    metadata = resource.Body('metadata', type=list)
    #: The project id which the zone belongs to
    project_id = resource.Body('project_id')
    #: Current status of the zone import
    status = resource.Body('status')
    #: Timestamp when the zone was last updated
    updated_at = resource.Body('updated_at')
    #: Version of the resource
    version = resource.Body('version', type=int)
    #: ID for the zone that was created by this import
    zone_id = resource.Body('zone_id')

    @classmethod
    def _transform_create_request(
        cls,
        session: adapter.Adapter,
        request: resource._Request,
        *,
        microversion: str | None,
    ) -> resource._Request:
        request.body = None
        request.headers['content-type'] = 'text/dns'
        return request



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class SfcPortChain(resource.Resource):
    resource_key = 'port_chain'
    resources_key = 'port_chains'
    base_path = '/sfc/port_chains'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'description',
        'name',
        'project_id',
        'tenant_id',
    )

    # Properties
    #: Human-readable description for the resource.
    description = resource.Body('description')
    #: Human-readable name of the resource. Default is an empty string.
    name = resource.Body('name')
    #: List of port-pair-group UUIDs.
    port_pair_groups = resource.Body('port_pair_groups', type=list)
    #: List of flow-classifier UUIDs.
    flow_classifiers = resource.Body('flow_classifiers', type=list)
    #: A dictionary of chain parameters, correlation values can be
    #: mpls and nsh, symmetric can be True or False.
    chain_parameters = resource.Body('chain_parameters', type=dict)
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any

from keystoneauth1 import adapter
import requests

from openstack import exceptions
from openstack import resource


class SecretConsumer(resource.Resource):
    resources_key = 'consumers'
    base_path = '/secrets/%(secret_id)s/consumers'

    allow_create = True
    allow_delete = True
    allow_list = True

    # DELETE on this endpoint uses a body and does not target a specific id
    requires_id = False

    # URI parameters
    secret_id = resource.URI('secret_id')

    service = resource.Body('service')
    resource_type = resource.Body('resource_type')
    resource_id = resource.Body('resource_id')

    def _raw_delete(
        self,
        session: adapter.Adapter,
        microversion: str | None = None,
        **attrs: Any,
    ) -> requests.Response:
        """Custom raw_delete method for Barbican consumers.

        Barbican requires DELETE requests to include a JSON body with
        service, resource_type, and resource_id fields.
        """
        if not self.allow_delete:
            raise exceptions.MethodNotSupported(self, 'delete')

        request = self._prepare_request(**attrs)
        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)

        # Barbican expects JSON body with service/resource_type/resource_id
        body = self._prepare_request_body(prepend_key=False)

        return session.delete(
            request.url,
            json=body,
            headers=request.headers,
            microversion=microversion,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import quota
from openstack.tests.unit import base

IDENTIFIER = "IDENTIFIER"
EXAMPLE = {
    "zones": 10,
    "zone_recordsets": 500,
    "zone_records": 500,
    "recordset_records": 20,
    "api_export_size": 1000,
}


class TestQuota(base.TestCase):
    def test_basic(self):
        sot = quota.Quota()
        self.assertIsNone(sot.resources_key)
        self.assertIsNone(sot.resource_key)
        self.assertEqual("/quotas", sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.commit_method, "PATCH")

    def test_make_it(self):
        sot = quota.Quota(project='FAKE_PROJECT', **EXAMPLE)
        self.assertEqual(EXAMPLE['zones'], sot.zones)
        self.assertEqual(EXAMPLE['zone_recordsets'], sot.zone_recordsets)
        self.assertEqual(EXAMPLE['zone_records'], sot.zone_records)
        self.assertEqual(EXAMPLE['recordset_records'], sot.recordset_records)
        self.assertEqual(EXAMPLE['api_export_size'], sot.api_export_size)
        self.assertEqual('FAKE_PROJECT', sot.project)

    def test_prepare_request(self):
        body = {'id': 'ABCDEFGH', 'zones': 20}
        quota_obj = quota.Quota(**body)
        response = quota_obj._prepare_request()
        self.assertNotIn('id', response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.orchestration.v1 import software_deployment
from openstack.tests.unit import base


FAKE = {
    'id': 'ce8ae86c-9810-4cb1-8888-7fb53bc523bf',
    'action': 'CREATE',
    'config_id': 'CONFIG ID',
    'creation_time': '2015-03-09T12:15:57',
    'server_id': 'FAKE_SERVER',
    'stack_user_project_id': 'ANOTHER PROJECT',
    'status': 'IN_PROGRESS',
    'status_reason': 'Why are we here?',
    'input_values': {'foo': 'bar'},
    'output_values': {'baz': 'zoo'},
    'updated_time': '2015-03-09T12:15:57',
}


class TestSoftwareDeployment(base.TestCase):
    def test_basic(self):
        sot = software_deployment.SoftwareDeployment()
        self.assertEqual('software_deployment', sot.resource_key)
        self.assertEqual('software_deployments', sot.resources_key)
        self.assertEqual('/software_deployments', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = software_deployment.SoftwareDeployment(**FAKE)
        self.assertEqual(FAKE['id'], sot.id)
        self.assertEqual(FAKE['action'], sot.action)
        self.assertEqual(FAKE['config_id'], sot.config_id)
        self.assertEqual(FAKE['creation_time'], sot.created_at)
        self.assertEqual(FAKE['server_id'], sot.server_id)
        self.assertEqual(
            FAKE['stack_user_project_id'], sot.stack_user_project_id
        )
        self.assertEqual(FAKE['input_values'], sot.input_values)
        self.assertEqual(FAKE['output_values'], sot.output_values)
        self.assertEqual(FAKE['status'], sot.status)
        self.assertEqual(FAKE['status_reason'], sot.status_reason)
        self.assertEqual(FAKE['updated_time'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import domain_config
from openstack.tests.unit import base


EXAMPLE = {
    'identity': {
        'driver': 'ldap',
    },
    'ldap': {
        'url': 'ldap://myldap.com:389/',
        'user_tree_dn': 'ou=Users,dc=my_new_root,dc=org',
    },
}


class TestDomainConfig(base.TestCase):
    def test_basic(self):
        sot = domain_config.DomainConfig()
        self.assertEqual('config', sot.resource_key)
        self.assertEqual('/domains/%(domain_id)s/config', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = domain_config.DomainConfig(**EXAMPLE)
        self.assertIsInstance(sot.identity, domain_config.DomainConfigDriver)
        self.assertEqual(EXAMPLE['identity']['driver'], sot.identity.driver)
        self.assertIsInstance(sot.ldap, domain_config.DomainConfigLDAP)
        self.assertEqual(EXAMPLE['ldap']['url'], sot.ldap.url)
        self.assertEqual(
            EXAMPLE['ldap']['user_tree_dn'],
            sot.ldap.user_tree_dn,
        )



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import glob
import json
import os
from typing import Any
import urllib

import requests
import yaml

from openstack.config import _util
from openstack import exceptions

_VENDORS_PATH = os.path.dirname(os.path.realpath(__file__))
_VENDOR_DEFAULTS: dict[str, dict[str, Any]] = {}
_WELL_KNOWN_PATH = "{scheme}://{netloc}/.well-known/openstack/api"


def _get_vendor_defaults() -> dict[str, dict[str, Any]]:
    global _VENDOR_DEFAULTS

    if not _VENDOR_DEFAULTS:
        for vendor in glob.glob(os.path.join(_VENDORS_PATH, '*.yaml')):
            with open(vendor) as f:
                vendor_data = yaml.safe_load(f)
                _VENDOR_DEFAULTS[vendor_data['name']] = vendor_data['profile']

        for vendor in glob.glob(os.path.join(_VENDORS_PATH, '*.json')):
            with open(vendor) as f:
                vendor_data = json.load(f)
                _VENDOR_DEFAULTS[vendor_data['name']] = vendor_data['profile']

    return _VENDOR_DEFAULTS


def get_profile(profile_name: str) -> dict[str, Any] | None:
    vendor_defaults = _get_vendor_defaults()
    if profile_name in vendor_defaults:
        return vendor_defaults[profile_name].copy()

    profile_url = urllib.parse.urlparse(profile_name)
    if not profile_url.netloc:
        # This isn't a url, and we already don't have it.
        return None

    well_known_url = _WELL_KNOWN_PATH.format(
        scheme=profile_url.scheme,
        netloc=profile_url.netloc,
    )
    response = requests.get(well_known_url, timeout=10)
    if not response.ok:
        raise exceptions.ConfigException(
            f"{profile_name} is a remote profile that could not be fetched: "
            f"{response.status_code} {response.reason}"
        )
        vendor_defaults[profile_name] = None
        return

    vendor_data = response.json()
    name = vendor_data['name']
    # Merge named and url cloud config, but make named config override the
    # config from the cloud so that we can supply local overrides if needed.
    profile = _util.merge_clouds(
        vendor_data['profile'], vendor_defaults.get(name, {})
    )
    # If there is (or was) a profile listed in a named config profile, it
    # might still be here. We just merged in content from a URL though, so
    # pop the key to prevent doing it again in the future.
    profile.pop('profile', None)
    # Save the data under both names so we don't reprocess this, no matter
    # how we're called.
    vendor_defaults[profile_name] = profile
    vendor_defaults[name] = profile

    return profile



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.compute.v2 import server_group
from openstack.tests.unit import base


EXAMPLE = {
    'id': 'IDENTIFIER',
    'name': 'test',
    'members': ['server1', 'server2'],
    'metadata': {},
    'policies': ['anti-affinity'],
    'rules': {
        'max_server_per_host': 5,
    },
}


class TestServerGroup(base.TestCase):
    def test_basic(self):
        sot = server_group.ServerGroup()
        self.assertEqual('server_group', sot.resource_key)
        self.assertEqual('server_groups', sot.resources_key)
        self.assertEqual('/os-server-groups', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual(
            {
                "all_projects": "all_projects",
                "limit": "limit",
                "marker": "marker",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = server_group.ServerGroup(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['members'], sot.member_ids)
        self.assertEqual(EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(EXAMPLE['policies'], sot.policies)
        self.assertEqual(EXAMPLE['rules'], sot.rules)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.baremetal.v1 import deploy_templates
from openstack.tests.unit import base


FAKE = {
    "created_at": "2016-08-18T22:28:48.643434+11:11",
    "extra": {},
    "links": [
        {
            "href": """http://10.60.253.180:6385/v1/deploy_templates
                    /bbb45f41-d4bc-4307-8d1d-32f95ce1e920""",
            "rel": "self",
        },
        {
            "href": """http://10.60.253.180:6385/deploy_templates
                   /bbb45f41-d4bc-4307-8d1d-32f95ce1e920""",
            "rel": "bookmark",
        },
    ],
    "name": "CUSTOM_HYPERTHREADING_ON",
    "steps": [
        {
            "args": {
                "settings": [{"name": "LogicalProc", "value": "Enabled"}]
            },
            "interface": "bios",
            "priority": 150,
            "step": "apply_configuration",
        }
    ],
    "updated_at": None,
    "uuid": "bbb45f41-d4bc-4307-8d1d-32f95ce1e920",
}


class DeployTemplates(base.TestCase):
    def test_basic(self):
        sot = deploy_templates.DeployTemplate()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('deploy_templates', sot.resources_key)
        self.assertEqual('/deploy_templates', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

    def test_instantiate(self):
        sot = deploy_templates.DeployTemplate(**FAKE)
        self.assertEqual(FAKE['steps'], sot.steps)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['extra'], sot.extra)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)
        self.assertEqual(FAKE['uuid'], sot.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class QoSRuleType(resource.Resource):
    resource_key = 'rule_type'
    resources_key = 'rule_types'
    base_path = '/qos/rule-types'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'type',
        'drivers',
        'all_rules',
        'all_supported',
    )

    # Properties
    #: QoS rule type name.
    type = resource.Body('type', alternate_id=True)
    #: List of QoS backend drivers supporting this QoS rule type
    drivers = resource.Body('drivers', type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.key_manager.v1 import secret_consumer
from openstack.tests.unit import base


EXAMPLE = {
    'secret_id': 'sid',
    'service': 'svc',
    'resource_type': 'type',
    'resource_id': 'rid',
}


class TestSecretConsumer(base.TestCase):
    def test_basic(self):
        sot = secret_consumer.SecretConsumer()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('consumers', sot.resources_key)
        self.assertEqual('/secrets/%(secret_id)s/consumers', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.requires_id)

    def test_make_it(self):
        sot = secret_consumer.SecretConsumer(**EXAMPLE)
        self.assertEqual(EXAMPLE['secret_id'], sot.secret_id)
        self.assertEqual(EXAMPLE['service'], sot.service)
        self.assertEqual(EXAMPLE['resource_type'], sot.resource_type)
        self.assertEqual(EXAMPLE['resource_id'], sot.resource_id)

    def test_delete_with_body(self):
        sot = secret_consumer.SecretConsumer(
            secret_id="sid",
            service="svc",
            resource_type="type",
            resource_id="rid",
        )
        session = mock.Mock(spec=adapter.Adapter)
        session.default_microversion = None
        delete_resp = mock.Mock()
        delete_resp.status_code = 204
        delete_resp.headers = {}
        session.delete.return_value = delete_resp

        sot._raw_delete(session)

        session.delete.assert_called_once()
        _, kwargs = session.delete.call_args
        self.assertEqual(
            {"service": "svc", "resource_type": "type", "resource_id": "rid"},
            kwargs["json"],
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, Self

from keystoneauth1 import adapter

from openstack import resource


class Workflow(resource.Resource):
    resource_key = 'workflow'
    resources_key = 'workflows'
    base_path = '/workflows'

    # capabilities
    allow_create = True
    allow_commit = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'marker', 'limit', 'sort_keys', 'sort_dirs', 'fields'
    )

    #: The name of this Workflow
    name = resource.Body("name")
    #: The inputs for this Workflow
    input = resource.Body("input")
    #: A Workflow definition using the Mistral v2 DSL
    definition = resource.Body("definition")
    #: A list of values associated with a workflow that users can use
    #: to group workflows by some criteria
    # TODO(briancurtin): type=list
    tags = resource.Body("tags")
    #: Can be either "private" or "public"
    scope = resource.Body("scope")
    #: The ID of the associated project
    project_id = resource.Body("project_id")
    #: The time at which the workflow was created
    created_at = resource.Body("created_at")
    #: The time at which the workflow was created
    updated_at = resource.Body("updated_at")

    def _request_kwargs(
        self,
        prepend_key: bool = True,
        base_path: str | None = None,
    ) -> dict[str, Any]:
        request = self._prepare_request(
            requires_id=False, prepend_key=prepend_key, base_path=base_path
        )

        headers = {"Content-Type": 'text/plain'}
        kwargs = {
            "data": self.definition,
        }

        scope = f"?scope={self.scope}"
        uri = request.url + scope

        request.headers.update(headers)
        return dict(url=uri, json=None, headers=request.headers, **kwargs)

    # TODO(stephenfin): Migrate to _transform_create_request once _Request
    # gains a 'data' field for non-JSON request bodies. Currently the override
    # sends the workflow definition as text/plain via data=self.definition with
    # json=None, which cannot be expressed through the standard request object.
    def create(
        self,
        session: adapter.Adapter,
        prepend_key: bool = True,
        base_path: str | None = None,
        **params: Any,
    ) -> Self:
        kwargs = self._request_kwargs(
            prepend_key=prepend_key, base_path=base_path
        )
        response = session.post(**kwargs)
        self._translate_response(response, has_body=False)
        return self

    def commit(
        self,
        session: adapter.Adapter,
        prepend_key: bool = True,
        has_body: bool = True,
        retry_on_conflict: bool | None = None,
        base_path: str | None = None,
        *,
        microversion: str | None = None,
        **kwargs: Any,
    ) -> Self:
        request_kwargs = self._request_kwargs(
            prepend_key=prepend_key, base_path=base_path
        )
        response = session.put(**request_kwargs)
        self._translate_response(response, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.shared_file_system.v2 import share_type
from openstack.tests.unit import base


IDENTIFIER = '52a6d881-ba4e-4c50-aa0c-041e4e3756aa'
EXAMPLE = {
    "required_extra_specs": {"driver_handles_share_servers": "True"},
    "share_type_access:is_public": True,
    "extra_specs": {"driver_handles_share_servers": "True"},
    "id": IDENTIFIER,
    "name": "default-share-type",
    "is_default": True,
    "description": "manila share type",
}


class TestShareType(base.TestCase):
    def test_basic(self):
        types = share_type.ShareType()
        self.assertEqual('share_types', types.resources_key)
        self.assertEqual('/types', types.base_path)
        self.assertTrue(types.allow_list)
        self.assertTrue(types.allow_fetch)
        self.assertTrue(types.allow_commit)
        self.assertTrue(types.allow_delete)
        self.assertTrue(types.allow_create)
        self.assertFalse(types.allow_head)

        self.assertDictEqual(
            {
                'is_public': 'is_public',
                'extra_specs': 'extra_specs',
                'limit': 'limit',
                'marker': 'marker',
            },
            types._query_mapping._mapping,
        )

    def test_share_type(self):
        types = share_type.ShareType(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], types.id)
        self.assertEqual(EXAMPLE['name'], types.name)
        self.assertEqual(EXAMPLE['extra_specs'], types.extra_specs)
        self.assertEqual(
            EXAMPLE['required_extra_specs'], types.required_extra_specs
        )
        self.assertEqual(EXAMPLE['description'], types.description)
        self.assertEqual(EXAMPLE['is_default'], types.is_default)
        self.assertEqual(
            EXAMPLE['share_type_access:is_public'], types.is_public
        )

    def test_set_extra_specs(self):
        sot = share_type.ShareType(**EXAMPLE)
        specs = {'a': 'b', 'c': 'd'}

        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {'extra_specs': specs}
        sess = mock.Mock()
        sess.post.return_value = response

        result = sot.set_extra_specs(sess, **specs)

        sess.post.assert_called_once_with(
            f'types/{IDENTIFIER}/extra_specs',
            json={"extra_specs": specs},
            microversion=sess.default_microversion,
        )
        self.assertIsInstance(result, share_type.ShareType)
        self.assertEqual(specs, result.extra_specs)

    def test_delete_extra_specs(self):
        sot = share_type.ShareType(**EXAMPLE)

        sess = mock.Mock()
        response = mock.Mock()
        response.status_code = 200
        sess.delete.return_value = response

        sot.delete_extra_specs_property(sess, 'a')

        sess.delete.assert_called_once_with(
            f'types/{IDENTIFIER}/extra_specs/a',
            microversion=sess.default_microversion,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.tests.unit import base


class TestProxyBase(base.TestCase):
    def setUp(self):
        super().setUp()
        self.session = mock.Mock()

    def _verify(
        self,
        mock_method,
        test_method,
        *,
        method_args=None,
        method_kwargs=None,
        method_result=None,
        expected_args=None,
        expected_kwargs=None,
        expected_result=None,
    ):
        with mock.patch(mock_method) as mocked:
            mocked.return_value = expected_result
            if any(
                [
                    method_args,
                    method_kwargs,
                    expected_args,
                    expected_kwargs,
                ]
            ):
                method_args = method_args or ()
                method_kwargs = method_kwargs or {}
                expected_args = expected_args or ()
                expected_kwargs = expected_kwargs or {}

                if method_result:
                    self.assertEqual(
                        method_result,
                        test_method(*method_args, **method_kwargs),
                    )
                else:
                    self.assertEqual(
                        expected_result,
                        test_method(*method_args, **method_kwargs),
                    )

                # Check how the mock was called in detail
                called_args, called_kwargs = mocked.call_args
                self.assertEqual(expected_args, list(called_args))

                # NOTE(gtema): if base_path is not in expected_kwargs or empty
                # exclude it from the comparison, since some methods might
                # still invoke method with None value
                base_path = expected_kwargs.get('base_path', None)
                if base_path is None:
                    expected_kwargs.pop('base_path', None)
                    called_kwargs.pop('base_path', None)
                # ditto for paginated
                paginated = expected_kwargs.get('paginated', None)
                if paginated is None:
                    expected_kwargs.pop('paginated', None)
                    called_kwargs.pop('paginated', None)
                # and ignore_missing
                ignore_missing = expected_kwargs.get('ignore_missing', None)
                if ignore_missing is None:
                    expected_kwargs.pop('ignore_missing', None)
                    called_kwargs.pop('ignore_missing', None)

                self.assertDictEqual(expected_kwargs, called_kwargs)
            else:
                self.assertEqual(expected_result, test_method())
                mocked.assert_called_with(test_method.__self__)

    def verify_create(
        self,
        test_method,
        resource_type,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        expected_result="result",
        mock_method="openstack.proxy.Proxy._create",
    ):
        if method_args is None:
            method_args = []
        if method_kwargs is None:
            method_kwargs = {"x": 1, "y": 2, "z": 3}
        if expected_args is None:
            expected_args = method_args.copy()
        if expected_kwargs is None:
            expected_kwargs = method_kwargs.copy()
        expected_kwargs["base_path"] = base_path

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
            expected_result=expected_result,
        )

    def verify_delete(
        self,
        test_method,
        resource_type,
        ignore_missing=True,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.proxy.Proxy._delete",
    ):
        if method_args is None:
            method_args = ['resource_id']
        if method_kwargs is None:
            method_kwargs = {}
        method_kwargs["ignore_missing"] = ignore_missing
        if expected_args is None:
            expected_args = method_args.copy()
        if expected_kwargs is None:
            expected_kwargs = method_kwargs.copy()

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_get(
        self,
        test_method,
        resource_type,
        requires_id=False,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.proxy.Proxy._get",
    ):
        if method_args is None:
            method_args = ['resource_id']
        if method_kwargs is None:
            method_kwargs = {}
        if expected_args is None:
            expected_args = method_args.copy()
        if expected_kwargs is None:
            expected_kwargs = method_kwargs.copy()

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_get_overrided(self, proxy, resource_type, patch_target):
        with mock.patch(patch_target, autospec=True) as res:
            proxy._get_resource = mock.Mock(return_value=res)
            proxy._get(resource_type)
            res.fetch.assert_called_once_with(
                proxy,
                requires_id=True,
                base_path=None,
                error_message=mock.ANY,
                skip_cache=False,
            )

    def verify_head(
        self,
        test_method,
        resource_type,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.proxy.Proxy._head",
    ):
        if method_args is None:
            method_args = ['resource_id']
        if method_kwargs is None:
            method_kwargs = {}
        expected_args = expected_args or method_args.copy()
        expected_kwargs = expected_kwargs or method_kwargs.copy()

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_find(
        self,
        test_method,
        resource_type,
        name_or_id='resource_name',
        ignore_missing=True,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.proxy.Proxy._find",
    ):
        method_args = [name_or_id] + (method_args or [])
        method_kwargs = method_kwargs or {}
        method_kwargs["ignore_missing"] = ignore_missing
        expected_args = expected_args or method_args.copy()
        expected_kwargs = expected_kwargs or method_kwargs.copy()

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_list(
        self,
        test_method,
        resource_type,
        paginated=None,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        mock_method="openstack.proxy.Proxy._list",
    ):
        if method_args is None:
            method_args = []
        if method_kwargs is None:
            method_kwargs = {}
        if paginated is not None:
            method_kwargs["paginated"] = paginated
        if expected_args is None:
            expected_args = method_args.copy()
        if expected_kwargs is None:
            expected_kwargs = method_kwargs.copy()
        if base_path is not None:
            expected_kwargs["base_path"] = base_path

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_update(
        self,
        test_method,
        resource_type,
        base_path=None,
        *,
        method_args=None,
        method_kwargs=None,
        expected_args=None,
        expected_kwargs=None,
        expected_result="result",
        mock_method="openstack.proxy.Proxy._update",
    ):
        if method_args is None:
            method_args = ['resource_id']
        if method_kwargs is None:
            method_kwargs = {"x": 1, "y": 2, "z": 3}
        method_kwargs["base_path"] = base_path
        if expected_args is None:
            expected_args = method_args.copy()
        if expected_kwargs is None:
            expected_kwargs = method_kwargs.copy()

        self._verify(
            mock_method,
            test_method,
            method_args=method_args,
            method_kwargs=method_kwargs,
            expected_args=[resource_type, *expected_args],
            expected_kwargs=expected_kwargs,
        )

    def verify_wait_for_status(
        self,
        test_method,
        mock_method="openstack.resource.wait_for_status",
        **kwargs,
    ):
        self._verify(mock_method, test_method, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Operations with the provision state in the Bare Metal service.
"""


def manage_and_inspect_node(conn, uuid):
    node = conn.baremetal.find_node(uuid)
    print('Before:', node.provision_state)
    conn.baremetal.set_node_provision_state(node, 'manage')
    conn.baremetal.wait_for_nodes_provision_state([node], 'manageable')
    conn.baremetal.set_node_provision_state(node, 'inspect')
    res = conn.baremetal.wait_for_nodes_provision_state([node], 'manageable')
    print('After:', res[0].provision_state)


def provide_node(conn, uuid):
    node = conn.baremetal.find_node(uuid)
    print('Before:', node.provision_state)
    conn.baremetal.set_node_provision_state(node, 'provide')
    res = conn.baremetal.wait_for_nodes_provision_state([node], 'available')
    print('After:', res[0].provision_state)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_stack
----------------------------------

Functional tests for stack methods.
"""

import tempfile

from openstack import exceptions
from openstack.tests.functional import base

simple_template = """heat_template_version: 2014-10-16
parameters:
  length:
    type: number
    default: 10

resources:
  my_rand:
    type: OS::Heat::RandomString
    properties:
      length: {get_param: length}
outputs:
  rand:
    value:
      get_attr: [my_rand, value]
"""

root_template = """heat_template_version: 2014-10-16
parameters:
  length:
    type: number
    default: 10
  count:
    type: number
    default: 5

resources:
  my_rands:
    type: OS::Heat::ResourceGroup
    properties:
      count: {get_param: count}
      resource_def:
        type: My::Simple::Template
        properties:
          length: {get_param: length}
outputs:
  rands:
    value:
      get_attr: [my_rands, attributes, rand]
"""

environment = """
resource_registry:
  My::Simple::Template: %s
"""

validate_template = """heat_template_version: asdf-no-such-version """


class TestStack(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()
        if not self.user_cloud.has_service('orchestration'):
            self.skipTest('Orchestration service not supported by cloud')

    def _cleanup_stack(self):
        self.user_cloud.delete_stack(self.stack_name, wait=True)
        self.assertIsNone(self.user_cloud.get_stack(self.stack_name))

    def test_stack_validation(self):
        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(validate_template.encode('utf-8'))
        test_template.close()
        stack_name = self.getUniqueString('validate_template')
        self.assertRaises(
            exceptions.SDKException,
            self.user_cloud.create_stack,
            name=stack_name,
            template_file=test_template.name,
        )

    def test_stack_simple(self):
        test_template = tempfile.NamedTemporaryFile(delete=False)
        test_template.write(simple_template.encode('utf-8'))
        test_template.close()
        self.stack_name = self.getUniqueString('simple_stack')
        self.addCleanup(self._cleanup_stack)
        stack = self.user_cloud.create_stack(
            name=self.stack_name, template_file=test_template.name, wait=True
        )

        # assert expected values in stack
        self.assertEqual('CREATE_COMPLETE', stack['stack_status'])
        rand = stack['outputs'][0]['output_value']
        self.assertEqual(10, len(rand))

        # assert get_stack matches returned create_stack
        stack = self.user_cloud.get_stack(self.stack_name)
        self.assertEqual('CREATE_COMPLETE', stack['stack_status'])
        self.assertEqual(rand, stack['outputs'][0]['output_value'])

        # assert stack is in list_stacks
        stacks = self.user_cloud.list_stacks()
        stack_ids = [s['id'] for s in stacks]
        self.assertIn(stack['id'], stack_ids)

        # update with no changes
        stack = self.user_cloud.update_stack(
            self.stack_name, template_file=test_template.name, wait=True
        )

        # assert no change in updated stack
        self.assertEqual('UPDATE_COMPLETE', stack['stack_status'])
        rand = stack['outputs'][0]['output_value']
        self.assertEqual(rand, stack['outputs'][0]['output_value'])

        # update with changes
        stack = self.user_cloud.update_stack(
            self.stack_name,
            template_file=test_template.name,
            wait=True,
            length=12,
        )

        # assert changed output in updated stack
        stack = self.user_cloud.get_stack(self.stack_name)
        self.assertEqual('UPDATE_COMPLETE', stack['stack_status'])
        new_rand = stack['outputs'][0]['output_value']
        self.assertNotEqual(rand, new_rand)
        self.assertEqual(12, len(new_rand))

    def test_stack_nested(self):
        test_template = tempfile.NamedTemporaryFile(
            suffix='.yaml', delete=False
        )
        test_template.write(root_template.encode('utf-8'))
        test_template.close()

        simple_tmpl = tempfile.NamedTemporaryFile(suffix='.yaml', delete=False)
        simple_tmpl.write(simple_template.encode('utf-8'))
        simple_tmpl.close()

        env = tempfile.NamedTemporaryFile(suffix='.yaml', delete=False)
        expanded_env = environment % simple_tmpl.name
        env.write(expanded_env.encode('utf-8'))
        env.close()

        self.stack_name = self.getUniqueString('nested_stack')
        self.addCleanup(self._cleanup_stack)
        stack = self.user_cloud.create_stack(
            name=self.stack_name,
            template_file=test_template.name,
            environment_files=[env.name],
            wait=True,
        )

        # assert expected values in stack
        self.assertEqual('CREATE_COMPLETE', stack['stack_status'])
        rands = stack['outputs'][0]['output_value']
        self.assertEqual(['0', '1', '2', '3', '4'], sorted(rands.keys()))
        for rand in rands.values():
            self.assertEqual(10, len(rand))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.key_manager.v1 import secret_store as _secret_store
from openstack.tests.functional import base


class TestSecretStore(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()
        self.require_service('key-manager')

    def test_secret_store(self):
        """Test Secret Store operations"""
        key_manager = self.operator_cloud.key_manager

        # Test list secret stores
        secret_stores = list(key_manager.secret_stores())
        self.assertIsInstance(secret_stores, list)

        for store in secret_stores:
            self.assertIsInstance(store, _secret_store.SecretStore)
            self.assertIsNotNone(store.name)
            self.assertIsNotNone(store.status)

        # Test list secret stores with filters
        global_default_stores = list(
            key_manager.secret_stores(global_default=True)
        )
        self.assertIsInstance(global_default_stores, list)

        active_stores = list(key_manager.secret_stores(status="ACTIVE"))
        self.assertIsInstance(active_stores, list)

        # Test get global default secret store
        if global_default_stores:
            default_store = key_manager.get_global_default_secret_store()
            self.assertIsInstance(default_store, _secret_store.SecretStore)
            self.assertIsNotNone(default_store.name)
            self.assertTrue(default_store.global_default)

        # Test get preferred secret store
        if secret_stores:
            preferred_store = key_manager.get_preferred_secret_store()
            self.assertIsInstance(preferred_store, _secret_store.SecretStore)
            self.assertIsNotNone(preferred_store.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.identity.v3 import user
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'default_project_id': '1',
    'description': '2',
    'domain_id': '3',
    'email': '4',
    'enabled': True,
    'id': IDENTIFIER,
    'links': {'self': 'http://example.com/user1'},
    'name': '6',
    'password': '7',
    'password_expires_at': '8',
}


class TestUser(base.TestCase):
    def setUp(self):
        super().setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = 1
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.good_resp = mock.Mock()
        self.good_resp.body = None
        self.good_resp.json = mock.Mock(return_value=self.good_resp.body)
        self.good_resp.status_code = 204

    def test_basic(self):
        sot = user.User()
        self.assertEqual('user', sot.resource_key)
        self.assertEqual('users', sot.resources_key)
        self.assertEqual('/users', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

        self.assertDictEqual(
            {
                'domain_id': 'domain_id',
                'name': 'name',
                'password_expires_at': 'password_expires_at',
                'is_enabled': 'enabled',
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = user.User(**EXAMPLE)
        self.assertEqual(EXAMPLE['default_project_id'], sot.default_project_id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['email'], sot.email)
        self.assertTrue(sot.is_enabled)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['password'], sot.password)
        self.assertEqual(
            EXAMPLE['password_expires_at'], sot.password_expires_at
        )

    def test_update_password(self):
        sot = user.User(**EXAMPLE)
        resp = self.good_resp
        self.sess.post = mock.Mock(return_value=resp)

        sot.update_password(self.sess, current_password="orig", password="new")

        self.sess.post.assert_called_with(
            "users/IDENTIFIER/password",
            json={
                "user": {"password": "new", "original_password": "orig"},
            },
        )
